@@ -9,14 +9,16 @@ from neoload_cli_lib import tools
 from neoload_cli_lib import user_data, cli_exception
 from neoload_cli_lib.name_resolver import Resolver
 
-__endpoint = "v2/tests"
-__resolver = Resolver(__endpoint)
+import logging
+
+__endpoint = "/tests"
+__resolver = Resolver(__endpoint, rest_crud.base_endpoint_with_workspace)
 
 meta_key = 'settings id'
 
 
 @click.command()
-@click.argument('command', type=click.Choice(['ls', 'create', 'put', 'patch', 'delete', 'use'], case_sensitive=False),
+@click.argument('command', type=click.Choice(['ls', 'create', 'put', 'patch', 'delete', 'use', 'createorpatch'], case_sensitive=False),
                 required=False)
 @click.argument("name", type=str, required=False)
 @click.option('--rename', help="rename test settings")
@@ -25,7 +27,8 @@ meta_key = 'settings id'
 @click.option('--zone', 'controller_zone_id', help="controller zone and it default zone for the lg.")
 @click.option('--lgs', 'lg_zone_ids', help="precise how many lg and other zone if needed. by default we use one lg.")
 @click.option('--naming-pattern', 'naming_pattern', help="")
-def cli(command, name, rename, description, scenario, controller_zone_id, lg_zone_ids, naming_pattern):
+@click.option('--file', type=click.File('r'), help="Json file with the data to be sent to the API.")
+def cli(command, name, rename, description, scenario, controller_zone_id, lg_zone_ids, naming_pattern, file):
     """
     ls     # Lists test settings                                             .
     create # Create a new test settings                                      .
@@ -34,7 +37,9 @@ def cli(command, name, rename, description, scenario, controller_zone_id, lg_zon
     delete # Remove a test settings and all associated test results          .
     use    # Remember the test settings you want to work on. Example : neoload
     |        test-settings use MyTest ; neoload test-settings delete         .
+    createorpatch # Create a new test or patch an existing one; useful in CI .
     """
+    rest_crud.set_current_command()
     if not command:
         print("command is mandatory. Please see neoload tests-settings --help")
         return
@@ -47,9 +52,13 @@ def cli(command, name, rename, description, scenario, controller_zone_id, lg_zon
         tools.ls(name, is_id, __resolver)
         return
     elif command == "create":
-        id_created = create(create_json(name, description, scenario, controller_zone_id, lg_zone_ids, naming_pattern))
+        id_created = create(create_json(name, description, scenario, controller_zone_id, lg_zone_ids, naming_pattern, file))
         user_data.set_meta(meta_key, id_created)
         return
+    elif command == "createorpatch":
+        __id = create_or_patch(name, rename, description, scenario, controller_zone_id, lg_zone_ids, naming_pattern, file)
+        user_data.set_meta(meta_key, __id)
+        return
 
     __id = tools.get_id(name, __resolver, is_id)
 
@@ -61,10 +70,10 @@ def cli(command, name, rename, description, scenario, controller_zone_id, lg_zon
         __id = user_data.get_meta_required(meta_key)
 
     if command == "put":
-        put(__id, create_json(rename, description, scenario, controller_zone_id, lg_zone_ids, naming_pattern))
+        put(__id, create_json(rename, description, scenario, controller_zone_id, lg_zone_ids, naming_pattern, file))
         user_data.set_meta(meta_key, __id)
     elif command == "patch":
-        patch(__id, create_json(rename, description, scenario, controller_zone_id, lg_zone_ids, naming_pattern))
+        patch(__id, create_json(rename, description, scenario, controller_zone_id, lg_zone_ids, naming_pattern, file))
         user_data.set_meta(meta_key, __id)
     elif command == "delete":
         delete(__id)
@@ -72,7 +81,7 @@ def cli(command, name, rename, description, scenario, controller_zone_id, lg_zon
 
 
 def create(json_data):
-    rep = rest_crud.post(__endpoint, fill_default_fields(json_data))
+    rep = rest_crud.post(get_end_point(), fill_default_fields(json_data))
     return tools.get_id_and_print_json(rep)
 
 
@@ -82,21 +91,28 @@ def put(id_settings, json_data):
 
 
 def patch(id_settings, json_data):
-    rep = rest_crud.patch(get_end_point(id_settings), fill_default_fields(json_data))
+    rep = rest_crud.patch(get_end_point(id_settings), json_data)
     tools.get_id_and_print_json(rep)
 
 
 def delete(__id):
-    rep = tools.delete(__endpoint, __id, "settings")
+    rep = tools.delete(get_end_point(), __id, "settings")
     tools.print_json(rep.json())
     user_data.set_meta(meta_key, None)
 
 
-def get_end_point(id_test: str):
-    return __endpoint + "/" + id_test
+def get_end_point(id_test: str = None):
+    slash_id_test = '' if id_test is None else '/' + id_test
+    return rest_crud.base_endpoint_with_workspace() + '/tests' + slash_id_test
 
 
-def create_json(name, description, scenario, controller_zone_id, lg_zone_ids, naming_pattern):
+def create_json(name, description, scenario, controller_zone_id, lg_zone_ids, naming_pattern, file):
+    if file:
+        try:
+            return json.load(file)
+        except json.JSONDecodeError as err:
+            raise cli_exception.CliException('%s\nThis command requires a valid Json input.\n'
+                                             'Example: neoload test-settings create {"name":"TestName"}' % str(err))
     data = {}
     if name is not None:
         data['name'] = name
@@ -122,12 +138,6 @@ def manual_json(data):
         for field in ['name', 'description', 'scenarioName', 'controllerZoneId', 'testResultNamingPattern']:
             data[field] = input(field)
         data['lgZoneIds'] = parse_zone_ids(input("lgZoneIds"), data['controllerZoneId'])
-    else:
-        try:
-            data = json.loads(sys.stdin.read())
-        except json.JSONDecodeError as err:
-            raise cli_exception.CliException('%s\nThis command requires a valid Json input.\n'
-                                             'Example: neoload test-settings create {"name":"TestName"}' % str(err))
     return data
 
 
@@ -161,3 +171,17 @@ def fill_default_fields(json_data):
         ('lgZoneIds', default_lgs(json_data.get('lgZoneIds'), json_data.get('controllerZoneId')))
     ])
     return data
+
+
+def create_or_patch(name, rename, description, scenario, controller_zone_id, lg_zone_ids, naming_pattern, file):
+    is_id = tools.is_id(name)
+
+    __id = tools.get_id(name, __resolver, is_id, True)
+
+    if __id is None:
+        logging.info('creating test-settings: ' + str(name))
+        __id = create(create_json(name, description, scenario, controller_zone_id, lg_zone_ids, naming_pattern, file))
+    else:
+        logging.info('Patching test-settings: ' + str(__id))
+        patch(__id, create_json(rename, description, scenario, controller_zone_id, lg_zone_ids, naming_pattern, file))
+    return __id
